/*============================================================================
  File:     filestream.sql

  Summary:  This script illustrates configuring the filestream feature at an
			instance level, creating a database with a filestream filegroup,
			creating a table with a FILESTREAM column, and using DML statements
			against the data in the table.
			
			This also serves as a "setup script" for the .NET filestream access
			program, FileStreamInsert.

  Date:     August 2008

  SQL Server Version: 10.0.1600.22 (RTM)
------------------------------------------------------------------------------
  Written by Bob Beauchemin, SQLskills.com

  For more scripts and sample code, check out http://www.SQLskills.com

  This script is intended only as a supplement to SQL Server Jumpstart program. 
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- this is to enable at an engine level
-- NOTE: Filestream must also be configured at an OS level using 
-- SQL Server Configuration Manager (see BOL for details)
EXEC sp_configure 'filestream access level', '2'
RECONFIGURE
GO

USE master;
GO
IF EXISTS (
  SELECT * FROM sys.databases 
  WHERE name = N'FileStreamDB'
)
  DROP DATABASE FileStreamDB
GO

!! if not exist c:\Files mkdir C:\Files
GO

CREATE DATABASE FileStreamDB ON PRIMARY
  ( NAME = FileStreamDB_data, 
    FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL10.SQLDEV01\MSSQL\Data\FileStreamDB_data.mdf',
    SIZE = 5MB,
    MAXSIZE = 50MB, 
    FILEGROWTH = 15%),
FILEGROUP FileStreamGroup1 CONTAINS FILESTREAM
  ( NAME = FileStreamDBResumes, 
    FILENAME = N'C:\Files\Resumes')
LOG ON 
  ( NAME = 'FileStreamDB_log', 
    FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL10.SQLDEV01\MSSQL\Data\FileStreamDB_log.ldf',
    SIZE = 1MB, 
    MAXSIZE = 25MB, 
    FILEGROWTH = 5MB);
GO

USE FileStreamDB;
GO
IF OBJECT_ID('dbo.student', 'U') IS NOT NULL
  DROP TABLE dbo.student
GO

CREATE TABLE dbo.student
(student_id UNIQUEIDENTIFIER ROWGUIDCOL NOT NULL UNIQUE, 
  Name varchar(25),
  Resume varbinary(max) FILESTREAM);
GO

-- no file
USE FileStreamDB;
GO
INSERT INTO dbo.student
    VALUES (newid (), 'Alec', NULL);
GO

USE FileStreamDB;
GO
INSERT INTO dbo.student
    VALUES (newid (), 'Bob', CAST ('' as varbinary(max)));
INSERT INTO dbo.student
    VALUES (newid (), 'Fred', CAST ('' as varbinary(max)));
GO
    
USE FileStreamDB;
GO
INSERT INTO dbo.student
    VALUES (newid (), 'Cecille', 
      CAST ('My resume text' as varbinary(max)));
      
-- SELECT data
SELECT name, CAST(Resume as VARCHAR(MAX)) FROM dbo.student;
Go

-- UPDATE data
USE FileStreamDB;
GO
UPDATE dbo.student
SET Resume = CAST ('My updated resume text' as varbinary(max))
WHERE Name = 'Cecille';
GO

-- DELETE data
USE FileStreamDB;
GO
DELETE dbo.student
WHERE Name = 'Bob';
GO

UPDATE Student SET Resume = cast('' as varbinary(max)) WHERE Resume IS NULL
GO

-- Write method cannot be used
UPDATE Student SET Resume.Write(cast('resume' as varbinary(max)), 10, 7)
WHERE Name = 'Cecille';

-- PathName() is case sensitive
SELECT student.Resume.PathName()
  FROM dbo.student;
GO

-- PathName() with different parameter values
SELECT student.Resume.PathName(0)
  FROM dbo.student;
GO
SELECT student.Resume.PathName(1)
  FROM dbo.student;
GO
SELECT student.Resume.PathName(2)
  FROM dbo.student;
GO

CREATE TABLE t
    (
    Id int
    ,RowGuidColumn UNIQUEIDENTIFIER NOT NULL UNIQUE ROWGUIDCOL
    ,FileStreamColumn varbinary(max) FILESTREAM
    );
    
INSERT INTO t VALUES(1, newid(), CAST ('Some text' as varbinary(max)));
 

-- ROWGUID Columns in Views
-- SELECT FileStreamColumn.PathName() will succeed with View1, 
-- but fail with View2 because RowGuidColumn is not exposed.

-- Succeeds 
CREATE VIEW View1 
AS
SELECT RowGuidColumn
    ,FileStreamColumn
FROM t;

SELECT FileStreamColumn.PathName() FROM View1
GO


CREATE VIEW View2 
AS
SELECT FileStreamColumn
FROM t;
go

-- Fails because it is missing the RowGuidColumn
SELECT FileStreamColumn.PathName() FROM View2
GO
 

-- ROWGUID Columns in Derived Tables
-- In the following example the first statement succeeds. 
-- The second fails because RowGuidColumn is not exposed in the derived table.

-- Succeeds 

SELECT FileStreamColumn.PathName() 
FROM (
      SELECT FileStreamColumn, RowGuidColumn
      FROM t
      ) AS tab ;

-- Fails because it is missing the RowGuidColumn
SELECT FileStreamColumn.PathName()
FROM (
      SELECT FileStreamColumn 
      FROM t
      ) AS tab ;

CREATE TABLE t2 (
 id int,
 savedpath nvarchar(max) 
);

INSERT t2
SELECT id, FileStreamColumn.PathName() FROM t 

-- ROWGUID Columns in Scalar Subqueries
-- In the following example the first statement succeeds. 
-- The second fails because Function PathName is only valid on columns with the FILESTREAM attribute.

-- Succeeds 
SELECT t2.id, t.FileStreamColumn.PathName()
FROM t 
    INNER JOIN t2 
        ON t.id = t2.id; 

-- Fails
SELECT id, 
    (SELECT FileStreamColumn
    FROM t 
    WHERE t.id = t2.id
    ).PathName()  
FROM t2;

-- Works
SELECT id, 
    (SELECT FileStreamColumn.PathName()  
    FROM t 
    WHERE t.id = t2.id
    )
FROM t2;
 
-- Isolation levels

-- Not permitted at all if there are filestreams
ALTER DATABASE FILESTREAMDB SET ALLOW_SNAPSHOT_ISOLATION ON
GO

-- Not permitted at all if there are filestreams
ALTER DATABASE FILESTREAMDB SET READ_COMMITTED_SNAPSHOT ON
GO

SET TRANSACTION ISOLATION LEVEL REPEATABLE READ
SELECT name, CAST(Resume as VARCHAR(MAX)) FROM dbo.student;
go

SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED
SELECT name, CAST(Resume as VARCHAR(MAX)) FROM dbo.student;
go

SET TRANSACTION ISOLATION LEVEL SERIALIZABLE
begin tran
SELECT name, CAST(Resume as VARCHAR(MAX)) FROM dbo.student;
commit
go

SET TRANSACTION ISOLATION LEVEL READ COMMITTED
begin tran
UPDATE dbo.student
SET Resume = CAST ('12345' as varbinary(max))
WHERE Name = 'Cecille';
commit
go







